@@ -2,33 +2,37 @@
 #define __RMARIADB_MARIA_RESULT_SIMPLE__
 
 #include <boost/noncopyable.hpp>
-#include "MariaConnection.h"
+#include "DbConnection.h"
 #include "MariaBinding.h"
-#include "MariaResult.h"
+#include "DbResult.h"
+#include "MariaResultImpl.h"
 #include "MariaRow.h"
 #include "MariaTypes.h"
 #include "MariaUtils.h"
 
-class MariaResultSimple : boost::noncopyable, public MariaResult {
-  MariaConnectionPtr pConn_;
+class MariaResultSimple : boost::noncopyable, public MariaResultImpl {
+  DbResult* pRes_;
 
 public:
-  MariaResultSimple(MariaConnectionPtr conn);
+  MariaResultSimple(DbResult* res);
   ~MariaResultSimple();
 
 public:
   virtual void send_query(const std::string& sql);
   virtual void close();
 
-  virtual void bind(List params);
+  virtual void bind(const List& params);
 
-  virtual List column_info();
+  virtual List get_column_info();
 
   virtual List fetch(int n_max = -1);
 
-  virtual int rows_affected();
-  virtual int rows_fetched();
+  virtual int n_rows_affected();
+  virtual int n_rows_fetched();
   virtual bool complete();
+
+private:
+  void exec(const std::string& sql);
 };
 
 #endif
